@@ -1,13 +1,14 @@
 import shutil
-import queue
 import sqlite3
 import datetime
+import asyncio
+import queue
 from threadwrapper import *
 from filehandling import join_path, abs_main_dir
 from encryptedsocket import SC as ESC, SS as ESS
 from unencryptedsocket import SC as USC, SS as USS
 from easyrsa import *
-from omnitools import def_template, args, key_pair_format
+from omnitools import args, key_pair_format
 
 
 __ALL__ = ["SqlQueue"]
@@ -48,32 +49,23 @@ class SqlQueue(object):
             backup(_db)
             self.do_backup = False
             return connect_db(_db)
-        timeout_commit = 0
-        timeout_backup = 0
-        if self.auto_backup:
-            backup()
         conn = connect_db(_db)
+        if self.auto_backup:
+            conn = do_backup(conn)
+        prev_ts = time.time()
         while not self.terminate:
-            if self.do_commit:
+            time_diff = (time.time()-prev_ts)*1000
+            if self.do_commit or time_diff > self.timeout_commit:
                 commit_db(conn)
                 self.do_commit = False
-            elif self.do_backup:
+                prev_ts = time.time()
+            elif self.do_backup or (self.auto_backup and (time_diff > self.timeout_backup)):
                 conn = do_backup(conn)
+                prev_ts = time.time()
             elif self.sqlq.qsize() > 0:
                 tid, sql, data, row_factory = self.sqlq.get()
                 self.exc_result[tid] = self.__exc(conn, sql, data, row_factory)
-                self.sqlq.task_done()
-            else:
-                if (timeout_commit > self.timeout_commit) and len(self.exc_result) == 0:
-                    commit_db(conn)
-                    timeout_commit = 0
-                else:
-                    timeout_commit += 1
-                if self.auto_backup and (timeout_backup > self.timeout_backup) and len(self.exc_result) == 0:
-                    conn = do_backup(conn)
-                    timeout_backup = 0
-                else:
-                    timeout_backup += 1
+                # self.sqlq.task_done()
             time.sleep(1/1000)
         disconnect_db(conn)
         if self.auto_backup:
@@ -121,24 +113,39 @@ class SqlQueue(object):
             p(debug_info()[0])
             return e
 
-    def backup(self):
-        # To-do: client not working
-        self.do_backup = True
-        while self.do_backup:
+    def wait_task_to_finish(self, attribute_name, attribute_name2=None):
+        if not hasattr(self, attribute_name):
+            raise AttributeError("no {} in self".format(attribute_name))
+        if attribute_name2:
+            if not hasattr(self, attribute_name2):
+                raise AttributeError("no {} in self".format(attribute_name2))
+        setattr(self, attribute_name, True)
+        while getattr(self, attribute_name) if not attribute_name2 else not getattr(self, attribute_name2):
             time.sleep(1/1000)
+        return True
+
+    def wrap_under_is_server(self, task):
+        if self.is_server:
+            return self.wait_task_to_finish(task)
+        else:
+            return self.sc().request(command=task[3:], data=args())
+
+    def backup(self):
+        return self.wrap_under_is_server("do_backup")
 
     def commit(self):
-        # To-do: client not working
-        self.do_commit = True
-        while self.do_commit:
-            time.sleep(1/1000)
+        return self.wrap_under_is_server("do_commit")
 
     def stop(self):
-        self.terminate = True
-        while not self.worker_dead:
-            time.sleep(1/1000)
+        if self.is_server:
+            self.wait_task_to_finish("terminate", "worker_dead")
 
-    def __init__(self, server: bool = False, db: str = "", timeout_commit: int = 60*1000, auto_backup: bool = False, timeout_backup: int = 60*1000, depth: int = 2, export_functions = None) -> None:
+    def __init__(
+            self,
+            server: bool = False, db: str = "",
+            timeout_commit: int = 60*1000, auto_backup: bool = False, timeout_backup: int = 60*1000,
+            depth: int = 2, export_functions = None
+    ) -> None:
         self.is_server = server
         self.do_commit = False
         self.do_backup = False
@@ -157,53 +164,79 @@ class SqlQueue(object):
         if self.is_server:
             if not os.path.isabs(db):
                 db = join_path(abs_main_dir(depth=int(depth)), db)
-            self.timeout_commit = timeout_commit/20
+            self.timeout_commit = timeout_commit
             self.auto_backup = auto_backup
-            self.timeout_backup = timeout_backup/20
+            self.timeout_backup = timeout_backup
             self.sqlq = queue.Queue()
             self.sqlq_worker = threading.Thread(target=self.worker, args=(db,))
             self.sqlq_worker.daemon = True
             self.sqlq_worker.start()
             self.worker_dead = False
-            self.functions = dict(sql=self.sql)
+            self.functions = dict(sql=self.sql, backup=self.backup, commit=self.commit)
             if export_functions:
                 self.functions.update(export_functions=export_functions)
 
-    def _sql(self, tid: int, sql: str, data: tuple = (), row_factory: str = "row", _cmd: str = "sql") -> list:
+    async def _sql_async(self, tid: int, sql: str, data: tuple = (), row_factory: str = "row", _cmd: str = "sql", loop: asyncio.AbstractEventLoop = None) -> list:
         if self.is_server:
             if _cmd == "sql":
                 self.sqlq.put([tid, sql, data, row_factory])
-                while tid not in self.exc_result:
-                    time.sleep(1/1000)
-                    continue
-                try:
-                    return self.exc_result[tid]
-                finally:
-                    self.exc_result.pop(tid)
+                while True:
+                    try:
+                        return self.exc_result.pop(tid)
+                    except:
+                        await asyncio.sleep(2/1000, loop=loop)
             else:
                 try:
                     return self.functions[_cmd](*[sql, data, row_factory])
                 except Exception as e:
                     return e
         else:
-            return self.sc.request(command=_cmd, data=args(sql, data, row_factory))
+            return self.sc().request(command=_cmd, data=args(sql, data, row_factory))
 
-    def sql(self, sql: str, data: tuple = (), row_factory: str = "row",
+    def _sql(self, tid: int, sql: str, data: tuple = (), row_factory: str = "row", _cmd: str = "sql") -> list:
+        if self.is_server:
+            if _cmd == "sql":
+                self.sqlq.put([tid, sql, data, row_factory])
+                while True:
+                    try:
+                        return self.exc_result.pop(tid)
+                    except:
+                        time.sleep(2/1000)
+            else:
+                try:
+                    return self.functions[_cmd](*[sql, data, row_factory])
+                except Exception as e:
+                    return e
+        else:
+            return self.sc().request(command=_cmd, data=args(sql, data, row_factory))
+
+    def sql_(self, exc_type: str, sql: str, data: tuple = (), row_factory: str = "row",
             result: Any = None, key: Any = None, _cmd: str = "sql") -> list:
         if result is None:
             result = {}
-        key = key or 0
+        if key is None:
+            key = 0
         try:
             sql = " ".join([line.strip() for line in sql.splitlines() if not line.strip().startswith("--")]).strip()
         except:
             pass
 
-        def job(sql: str, data: tuple, row_factory: str, _cmd: str) -> list:
+        def job1():
             return self._sql(threading.get_ident(), sql, data, row_factory, _cmd)
 
-        threadwrapper = ThreadWrapper(threading.Semaphore(1))
-        threadwrapper.add(job=def_template(job, sql, data, row_factory, _cmd), result=result, key=key)
-        threadwrapper.wait()
+        def job2():
+            loop = asyncio.new_event_loop()
+            asyncio.set_event_loop(loop)
+            twa = ThreadWrapper_async(asyncio.Semaphore(1), loop=loop)
+            twa.add(self._sql_async(threading.get_ident(), sql, data, row_factory, _cmd), result, key)
+            twa.wait()
+
+        tw = ThreadWrapper(threading.Semaphore(1))
+        if exc_type == "async":
+            tw.add(job2)
+        else:
+            tw.add(job1, result, key)
+        tw.wait()
         if isinstance(result[key], Exception):
             if isinstance(result[key], sqlite3.OperationalError):
                 if "i/o" in str(result[key]).lower():
@@ -215,14 +248,19 @@ class SqlQueue(object):
             raise result[key]
         return result[key]
 
+    def sql(self, *args, **kwargs):
+        return self.sql_("", *args, **kwargs)
+
+    def sql_async(self, *args, **kwargs):
+        return self.sql_("async", *args, **kwargs)
+
 
 class SqlQueueE(SqlQueue):
-    def __init__(self, db_port: int = None, key_pair: key_pair_format = None, **kwargs) -> None:
+    def __init__(self, host = "127.199.71.10", db_port: int = None, key_pair: key_pair_format = None, **kwargs) -> None:
         if "db" in kwargs:
             if not os.path.isabs(kwargs["db"]):
                 kwargs["db"] = join_path(abs_main_dir(depth=int(kwargs["depth"])), kwargs["db"])
         super().__init__(**kwargs)
-        host = "127.199.71.10"
         port = db_port if db_port else 39292
         if self.is_server:
             if key_pair is None:
@@ -232,20 +270,21 @@ class SqlQueueE(SqlQueue):
             thread.daemon = True
             thread.start()
         else:
-            self.sc = ESC(host, port)
+            self.sc = lambda : ESC(host, port)
 
     def stop(self):
         super().stop()
-        self.ess.stop()
+        if self.is_server:
+            self.ess.stop()
+        return True
 
 
 class SqlQueueU(SqlQueue):
-    def __init__(self, db_port: int = None, **kwargs) -> None:
+    def __init__(self, host = "127.199.71.10", db_port: int = None, **kwargs) -> None:
         if "db" in kwargs:
             if not os.path.isabs(kwargs["db"]):
                 kwargs["db"] = join_path(abs_main_dir(depth=int(kwargs["depth"])), kwargs["db"])
         super().__init__(**kwargs)
-        host = "127.199.71.10"
         port = db_port if db_port else 39292
         if self.is_server:
             self.uss = USS(self.functions, host, port, True)
@@ -253,10 +292,12 @@ class SqlQueueU(SqlQueue):
             thread.daemon = True
             thread.start()
         else:
-            self.sc = USC(host, port)
+            self.sc = lambda : USC(host, port)
 
     def stop(self):
         super().stop()
-        self.uss.stop()
+        if self.is_server:
+            self.uss.stop()
+        return True
 
 
